# @(#)22  1.9 src/cmdline/staging/scripts/import_federations_and_partners.py, amws.cmdline, fim611, 060921a 9/18/06 16:18:35
# (C) Copyright International Business Machines Corporation 2005. All rights
# reserved.  U.S. Government Users Restricted Rights -- Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 
from fim_base import read_properties_file, commit_session, prepare_mgr_objs
from fed_utils import import_federation, import_partner, get_partner, get_partners
from domain_utils import is_domain_present
from com.tivoli.am.fim.logging import FIMLoggerFactory
from com.tivoli.am.fim.logging import FIMLogger
from com.tivoli.am.fim.logging import FIMLevel
from com.tivoli.am.fim.i18n.msgs import CmdLineMessages
from jarray import array
from java.lang import Object
from com.tivoli.am.fim.logging import FIMLoggerConstants
from com.tivoli.am.fim.middlemanager.ssps import FederationManager

# Set up FIMLogger
component_name = "StagingTools"
script_name = "import_federations_and_partners.py"
method_name = script_name
class_name = "com.tivoli.am.fim.cmdline." + script_name[0:-3]
FIMLoggerFactory.enableStdout(FIMLevel.ALL)
FIMLoggerFactory.setPreferredFIMLoggerImpl(FIMLoggerConstants.FIMLOGGER_BASE_IMPL)
log = FIMLoggerFactory.getFIMLogger(component_name, class_name, CmdLineMessages.CLASS_NAME)
import_props_filename = "_import.properties"
unix_slash = "/"
win_slash = "\\"

def usage():
    inserts = array([script_name],Object)
    log.logMessage(FIMLevel.INFO, method_name, \
        CmdLineMessages.usage_import_federations_and_partners, inserts)
    raise SystemExit

def import_partners(properties_directory, partner_dict, \
targ_fed_mgr, targ_sec_tok_mgr, fed_name, targ_props, src_fed):
    for partner_name, partner_props_file in partner_dict.items():
        partner_props = read_properties_file(properties_directory + unix_slash + partner_props_file)

        targ_fed = targ_fed_mgr.getFederationByDisplayName(fed_name, targ_sec_tok_mgr)

        # If the user properties file sets DisplayName, then that will be
        # the name of the new partner, so we have to set the name here
        # before checking whether a partner by that name exists. 
        if partner_props.has_key("DisplayName"):
            old_partner_name = partner_name
            new_partner_name = partner_props["DisplayName"]
        else:
            old_partner_name = new_partner_name = partner_name

        # Check to see if partner already exists
        targ_partner_dict = get_partners(targ_fed)
        if new_partner_name in targ_partner_dict.values():
            inserts = array([partner_name, fed_name], Object)
            log.logMessage(FIMLevel.INFO, method_name, \
                CmdLineMessages.partner_already_exists, inserts)
            continue

        inserts = array([old_partner_name, fed_name, targ_props["Domain"]], Object)
        log.logMessage(FIMLevel.INFO, method_name, \
            CmdLineMessages.importing_partner_in_domain, inserts)

        # Import the partner to the target federation with the supplied properties
        import_partner(src_fed, targ_fed, partner_props, old_partner_name)

        targ_partner_dict = get_partners(targ_fed)
        if new_partner_name not in targ_partner_dict.values():
            inserts = array([old_partner_name, fed_name, targ_props["Domain"]], Object)
            log.logMessage(FIMLevel.WARNING, method_name, \
                CmdLineMessages.unable_to_import_partner, inserts)
            continue
        else:
            inserts = array([new_partner_name, fed_name], Object)
            log.logMessage(FIMLevel.INFO, method_name, \
                CmdLineMessages.imported_partner_successfully, inserts)
        
if __name__ == 'main':

    if len(sys.argv) < 3:
        usage()

    src_props_file = sys.argv[0]
    targ_props_file = sys.argv[1]
    properties_directory = sys.argv[2]
    import_props_file = properties_directory + unix_slash + import_props_filename
    
    # Get the source federation manager
    src_props = read_properties_file(src_props_file)
    if not is_domain_present(src_props):
        raise SystemExit 
        
    (src_fed_mgr, src_sec_tok_mgr) = prepare_mgr_objs(src_props, \
        "import_federations_and_partners_src_fed")    

    # Get the target federation manager
    targ_props = read_properties_file(targ_props_file)
    if not is_domain_present(targ_props):
        raise SystemExit 
    (targ_fed_mgr, targ_sec_tok_mgr) = prepare_mgr_objs(targ_props, \
        "import_federations_and_partners_targ_fed")    

    # Get the properties defining which federations and partners will 
    # be imported
    import_props = read_properties_file(import_props_file)

    for fed_name in import_props.keys():
         
        if src_fed_mgr.hasFederationDisplayName(fed_name):
            src_fed = src_fed_mgr.getFederationByDisplayName(fed_name, src_sec_tok_mgr)
        else:
            # Cannot import a federation that doesn't exist in source domain
            inserts = array([fed_name, src_props["Domain"]], Object)
            log.logMessage(FIMLevel.INFO, method_name, \
                CmdLineMessages.federation_does_not_exist_in_domain, inserts)
            continue

        # Check that the right-hand side of the import properties is a PyList. 
        rhs = eval(import_props[fed_name])
        if not str(type(rhs)).find("PyList"):
            # Import properties file not in the right format
            inserts = array([fed_name, str(rhs), import_props_file], Object)
            log.logMessage(FIMLevel.WARNING, method_name, \
                CmdLineMessages.unable_to_parse_property_in_file, inserts)
            continue

        # If the user properties file sets DisplayName, then that will be
        # the name of the new federation, so we have to set the name here
        # before checking whether a federation by that name already exists. 
        fed_props_file = rhs[0]
        fed_props = read_properties_file(properties_directory + unix_slash 
            + fed_props_file)
        if fed_props.has_key("DisplayName"):
            old_fed_name = fed_name
            new_fed_name = fed_props["DisplayName"]
        else:
            old_fed_name = new_fed_name = fed_name

        if targ_fed_mgr.hasFederationDisplayName(new_fed_name):
            # Cannot import a federation with the same name
            inserts = array([new_fed_name, targ_props["Domain"]], Object)
            log.logMessage(FIMLevel.INFO, method_name, \
                CmdLineMessages.federation_already_exists_in_domain, inserts)
            # Try to import partners for an existing federation
            import_partners(properties_directory, rhs[1],\
                targ_fed_mgr, targ_sec_tok_mgr, new_fed_name, targ_props, src_fed)
            continue
        else:
            # Import federation
            inserts = array([old_fed_name, targ_props["Domain"]], Object)
            log.logMessage(FIMLevel.INFO, method_name, \
                CmdLineMessages.importing_federation_in_domain, inserts)
            import_federation(src_fed, targ_fed_mgr, targ_sec_tok_mgr, fed_props, old_fed_name)       
            
            if targ_fed_mgr.hasFederationDisplayName(fed_name):
                inserts = array([fed_name], Object)
                log.logMessage(FIMLevel.INFO, method_name, \
                    CmdLineMessages.imported_federation_successfully, inserts)
                # Import partners if federation was successfully imported
                import_partners(properties_directory, rhs[1], \
                    targ_fed_mgr, targ_sec_tok_mgr, fed_name, targ_props, src_fed)
            else:
                inserts = array([old_fed_name], Object)
                log.logMessage(FIMLevel.WARNING, method_name, \
                    CmdLineMessages.unable_to_import_federation, inserts)
                continue
        
    # Commit changes
    commit_session(targ_fed_mgr, None)
    
    
